#pragma once
#include "common.hpp"
/** @file */
#define SCALE_MAX 1.0 /**< Global maximum mouse scale factor*/
#define SCALE_MIN 0.1 /**< Global minimum mouse scale factor*/
 

/**
* Semantic Pointing for Object Picking in Complex 3D Environments
* This file is responsible the non-opengl related calculations in our implementation.
*/
class PickingHelper 
{
	public:

	/**
    * Use this to access the object instance.
    * @return The singleton object instance
    */
	static PickingHelper* getInstance();

	/**
     * PickingHelper class destructor 
	 */
	~PickingHelper();

	/**
	* Searches pickbuffer window for nearest objects and returns the nearest objects ID and its euclidean distance to the mouse cursor.
	* This method calls one of the private members @see checkNN_naive or @see checkNN_sophisticated, depending on the flag m_useNaiveSearch.
	* @param pickbuffer_window The pickbuffer window. It's supposed to contain object IDs > 0 whenever a pixel for a pickable object is drawn.
	* @param x_size Absolute width of the pickbuffer window.
	* @param y_size Absolute height of the pickbuffer window.
	* @param x_center Absolute mouse cursor x-location relative to pickbuffer_window origin.
	* @param y_center Absolute mouse cursor y-location relative to pickbuffer_window origin.
	* @param id Stores the object ID of the nearest object. Will be 0 if no object is found.
	* @param distance Stores the euclidean distance from the mouse cursor to the nearest object. Will be @see DBL_MAX if no object is found.
	*/
	void checkNN(const GLint* pickbuffer_window, const int x_size, const int y_size, const int x_center, const int y_center, int& id, double& distance);
	
	/**
	* Scaling function 
	* Calls either mouse_scale_linear or mouse_scale_inverse and returns the scale factor.
	* @see m_mouse_cutoff_distance
	* @see MOUSE_SCALING_CONSTANT
	* @param distance euclidean distance.
	* @return scaling factor in range [@see m_mouse_min_scale , @see m_mouse_max_scale].
	*/
	double scale_by_distance(double distance);

	/**
	* Uses Spring function to smooth out input. 
	* Spring Equation used: out = targetValue + (lastValue-targetValue)*exp(-deltaTime*springConstant);
	* @param lastValue the last value of teh variable.
	* @param targetValue The target value to smooth out.
	* @param deltaTime Time elapsed since last frame.
	* @param springConstant Spring Constant. Is 6 by default
	* @return smoothed out value.
	*/
	double springInput(double lastValue, double targetValue, double deltaTime, double springConstant);

	/**
	* Use this method to safely change the Cutoff - distance required for the linear scaling function.
	* @param deltaC desired difference in cutoff distance.
	* @return the new value. 
	*/
	double changeCutoffDistance(double deltaC)
	{
		m_mouse_cutoff_distance = glm::clamp(m_mouse_cutoff_distance + deltaC, 0.0, 2203.0); 
		m_pickingbuffer_window_width = 2 * m_mouse_cutoff_distance / sqrt(2.0f);
		return m_mouse_cutoff_distance;
	}; 

	/**
	* Use this method to safely change the minumum value of the scale function interval.
	* its range is limited by [ SCALE_MIN, m_mouse_max_scale]
	* @see SCALE_MIN
	* @see m_mouse_max_scale
	* @param delta_m desired difference in minimum interval.
	* @return the new value.
	*/
	double changeMinScale(double delta_m){m_mouse_min_scale = glm::clamp(m_mouse_min_scale + delta_m, SCALE_MIN, m_mouse_max_scale); return m_mouse_min_scale;};

	/**
	* Use this method to safely change the maximum value of the scale function interval.
	* its range is limited by [@see m_mouse_min_scale, @see SCALE_MAX]
	* @param delta_M desired difference in maximum interval.
	* @return the new value.
	*/
	double changeMaxScale(double delta_M){m_mouse_max_scale = glm::clamp(m_mouse_max_scale + delta_M, m_mouse_min_scale, SCALE_MAX); return m_mouse_max_scale;};

	/**
	* Use this method to safely change the exponent of the inverse scale function.
	* its range is limited by [0.1, 50.0]
	* @see m_mouse_scaling_exponent
	* @param delta_S desired difference in exponent.
	* @return the new value.
	*/
	double changeInverseExponent(double delta_S){m_mouse_scaling_exponent = glm::clamp(m_mouse_scaling_exponent + delta_S, 0.5, 50.0); return m_mouse_scaling_exponent;};

	/**
	* Toggles between usage of the naive- and the sophicticated pickbuffer search algorithm.
	* @return the boolean state.
	*/
	bool toggleNaiveSearch() {m_useNaiveSearch = !m_useNaiveSearch;return m_useNaiveSearch;};

	/**
	* Toggles between usage of the linear- and the inverse scaling function.
	* @return the boolean state.
	*/
	bool toggleLinearScaling() {m_useLinearScaling = !m_useLinearScaling;return m_useLinearScaling;};

	/**
	* Toggles wheather the bubble cursor should be displayed or the standard cursor.
	* @return the boolean state.
	*/
	bool toggleBubbleCursor() {m_showBubbleCursor = !m_showBubbleCursor;return m_showBubbleCursor;};

	/**
	* Toggles wheather the pickbuffer window should be displayed additionally.
	*/
	void togglePickbufferRectangle() {m_showPickbufferRectangle = !m_showPickbufferRectangle;};

	bool m_useLinearScaling; /**< scaling function switch*/
	bool m_useNaiveSearch; /**< search algorithm switch*/
	bool m_showPickbufferRectangle; /**< pickbuffer rectangle visualization switch*/
	bool m_showBubbleCursor; /**< bubble - / crosshair cursor switch*/
	int m_pickingbuffer_window_width; /**< Search width and height in pixel, assuming the window is quadratic */

private:
	
	/** 
    * The singleton pointer.
    */
	static PickingHelper* singletonPointer;

	/**
     * PickingHelper class constructor 
	 */
	PickingHelper();

	/**
	* Searches pickbuffer window for nearest objects and returns the nearest objects ID and its euclidean distance to the mouse cursor.
	* This implementation is a bit more sophisticated, than the naive approach. It searches for nearest objects by growing the search region starting from the mouse center until it hits an object.
	* This will increase search performance, as long as objects are nearby.
	* @param pickbuffer_window The pickbuffer window. It's supposed to contain object IDs > 0 whenever a pixel for a pickable object is drawn.
	* @param x_size Absolute width of the pickbuffer window.
	* @param y_size Absolute height of the pickbuffer window.
	* @param x_center Absolute mouse cursor x-location relative to pickbuffer_window origin.
	* @param y_center Absolute mouse cursor y-location relative to pickbuffer_window origin.
	* @param id Stores the object ID of the nearest object. Will be 0 if no object is found.
	* @param distance Stores the euclidean distance from the mouse cursor to the nearest object. Will be DBL_MAX @see DBL_MAX if no object is found.
	*/
	void checkNN_sophisticated(const GLint* pickbuffer_window, const int x_size, const int y_size, const int x_center, const int y_center, int& id, double& distance);

	/**
	* Searches pickbuffer window for nearest objects and returns the nearest objects ID and its euclidean distance to the mouse cursor.
	* This is a naive implementation. It runs through all entries in the pickbuffer. Due to its simplicity it is very fast for small pickbuffer windows.
	* @param pickbuffer_window The pickbuffer window. It's supposed to contain object IDs > 0 whenever a pixel for a pickable object is drawn.
	* @param x_size Absolute width of the pickbuffer window.
	* @param y_size Absolute height of the pickbuffer window.
	* @param x_center Absolute mouse cursor x-location relative to pickbuffer_window origin.
	* @param y_center Absolute mouse cursor y-location relative to pickbuffer_window origin.
	* @param id Stores the object ID of the nearest object. Will be 0 if no object is found.
	* @param distance Stores the euclidean distance from the mouse cursor to the nearest object. Will be DBL_MAX @see DBL_MAX if no object is found.
	*/
	void checkNN_naive(const GLint* pickbuffer_window, const int x_size, const int y_size, const int x_center, const int y_center, int& id, double& distance);

	/**
	* Linear scaling function
	* Returns a scaling factor based on the input euclidean distance.
	* @see m_mouse_max_scale
	* @see m_mouse_min_scale
	* @see m_mouse_cutoff_distance
	* @see MOUSE_SCALING_CONSTANT
	* @param distance euclidean distance.
	* @return scaling factor.
	*/
	double mouse_scale_linear(double distance);

	/**
	* Inverse scaling function 
	* Returns a scaling factor based on the input euclidean distance.
	* @see m_mouse_max_scale
	* @see m_mouse_min_scale
	* @see m_mouse_cutoff_distance
	* @see MOUSE_SCALING_CONSTANT
	* @param distance euclidean distance.
	* @return scaling factor.
	*/
	double mouse_scale_inverse(double distance);

	//VARIABLES
	double m_mouse_min_scale; /**< minimum scale function value*/
	double m_mouse_max_scale; /**< maximum scale function value*/
	double m_mouse_cutoff_distance; /**< Cutoff distance. If distance is higher than this threshold the maximum mouse scale factor will be output*/
	double m_mouse_scaling_exponent; /**< exponent in inverse scaling function*/
};

