#include "SoundInterface.hpp"
#include <fmod_errors.h>

SoundInterface::SoundInterface(void)
{
	initFMOD();
	initSounds();
	mIsMute = false;	
}


SoundInterface::~SoundInterface(void)
{
}

void SoundInterface::ERRCHECK(FMOD_RESULT result){    if (result != FMOD_OK)    { printf("FMOD error! (%d) %s\n", result, FMOD_ErrorString(result)); exit(-1); }}

bool SoundInterface::initFMOD(void)
{
	FMOD::System *system;
	FMOD_RESULT result;
	unsigned int version;
	int numdrivers;
	FMOD_SPEAKERMODE speakermode;
	FMOD_CAPS caps;
	char name[256];
	/*
	Create a System object and initialize.
	*/
	result = FMOD::System_Create(&system);
	ERRCHECK(result);
	result = system->getVersion(&version);
	ERRCHECK(result);
	if (version < FMOD_VERSION)
	{
		printf("Error! You are using an old version of FMOD %08x. This program requires %08x\n",
		version, FMOD_VERSION);
		return 0;
	}
	result = system->getNumDrivers(&numdrivers);
	ERRCHECK(result);
	if (numdrivers == 0)
	{
		result = system->setOutput(FMOD_OUTPUTTYPE_NOSOUND);
		ERRCHECK(result);
	}
	else
	{
		result = system->getDriverCaps(0, &caps, 0, &speakermode);
		ERRCHECK(result);
		/*
		Set the user selected speaker mode.
		*/
		result = system->setSpeakerMode(speakermode);
		ERRCHECK(result);
		if (caps & FMOD_CAPS_HARDWARE_EMULATED)
		{
			/*
			The user has the 'Acceleration' slider set to off! This is really bad
			for latency! You might want to warn the user about this.
			*/
			result = system->setDSPBufferSize(1024, 10);
			ERRCHECK(result);
		}
		result = system->getDriverInfo(0, name, 256, 0);
		ERRCHECK(result);
		if (strstr(name, "SigmaTel"))
		{
			/*
			Sigmatel sound devices crackle for some reason if the format is PCM 16bit.
			PCM floating point output seems to solve it.
			*/
			result = system->setSoftwareFormat(48000, FMOD_SOUND_FORMAT_PCMFLOAT, 0,0,
			FMOD_DSP_RESAMPLER_LINEAR);
			ERRCHECK(result);
		}
	}
	result = system->init(100, FMOD_INIT_NORMAL, 0);
	if (result == FMOD_ERR_OUTPUT_CREATEBUFFER)
	{
		/*
		Ok, the speaker mode selected isn't supported by this soundcard. Switch it
		back to stereo...
		*/
		result = system->setSpeakerMode(FMOD_SPEAKERMODE_STEREO);
		ERRCHECK(result);
		/*
		... and re-init.
		*/
		result = system->init(100, FMOD_INIT_NORMAL, 0);
	}
	ERRCHECK(result);
	this->mSystem = system;
	return true;
}


void SoundInterface::loadSound(const string& path, const SoundInterface::Sounds soundtype, const FMOD_MODE loop)
{
	FMOD_RESULT result;
	FMOD::Sound* sound = NULL;
	result = mSystem->createSound((pathtosounds + path).c_str(), loop, NULL, &sound);
	ERRCHECK(result);
	mSounds.insert(std::pair<SoundInterface::Sounds, FMOD::Sound*>(soundtype, sound));
}

void SoundInterface::initSounds(void)
{
	loadSound("MusicGood.mp3", SoundInterface::Sounds::MusicGood, FMOD_LOOP_NORMAL);
	loadSound("MusicBad.mp3", SoundInterface::Sounds::MusicBad, FMOD_LOOP_NORMAL);
	loadSound("SoundApple.mp3", SoundInterface::Sounds::SoundApple, FMOD_LOOP_OFF);
	loadSound("SoundHolyGrenadeRolling.mp3", SoundInterface::Sounds::SoundHolyGrenadeRolling, FMOD_LOOP_NORMAL);
	loadSound("SoundHolyGrenadeCrafting.mp3", SoundInterface::Sounds::SoundHolyGrenadeCrafting, FMOD_LOOP_NORMAL);	
	loadSound("SoundLightningPick.mp3", SoundInterface::Sounds::SoundLightningPick, FMOD_LOOP_NORMAL);
	loadSound("SoundLightningShoot.mp3", SoundInterface::Sounds::SoundLightningShoot, FMOD_LOOP_OFF);	
	loadSound("SoundPeopleShot.mp3", SoundInterface::Sounds::SoundPeopleShot, FMOD_LOOP_OFF);	
	loadSound("SoundHolyGrenadeFinished.mp3", SoundInterface::Sounds::SoundHolyGrenadeFinished, FMOD_LOOP_OFF);
	loadSound("SoundHolyGrenadeExplosion.mp3", SoundInterface::Sounds::SoundHolyGrenadeExplosion, FMOD_LOOP_OFF);
	loadSound("SoundGodRegret.mp3", SoundInterface::Sounds::SoundGodRegret, FMOD_LOOP_OFF);
	loadSound("SoundGodLaugh1.mp3", SoundInterface::Sounds::SoundGodLaugh1, FMOD_LOOP_OFF);
	loadSound("SoundGodLaugh2.mp3", SoundInterface::Sounds::SoundGodLaugh2, FMOD_LOOP_OFF);
	loadSound("SoundGodLaugh3.mp3", SoundInterface::Sounds::SoundGodLaugh3, FMOD_LOOP_OFF);
	loadSound("SoundGodLaugh4.mp3", SoundInterface::Sounds::SoundGodLaugh4, FMOD_LOOP_OFF);
	loadSound("SoundWarning.mp3", SoundInterface::Sounds::SoundWarning, FMOD_LOOP_OFF);	
	for(int i = 0; i < SoundInterface::Sounds::SOUNDS_COUNT ; i++)
	{
		mChannels[i] = 0;
		mVolumes[i] = 1.0f;
	}
	mVolumes[SoundInterface::Sounds::SoundLightningPick] = 0.8f;
	mVolumes[SoundInterface::Sounds::SoundLightningShoot] = 0.3f;
	mVolumes[SoundInterface::Sounds::SoundWarning] = 0.5f;	
	
}

void SoundInterface::play(SoundInterface::Sounds soundtype)
{
	FMOD_RESULT result;
	FMOD::Sound* sound;
	//FMOD::Channel* channel = 0;
	if(mSounds.count(soundtype)){
		sound = (*(mSounds.find(soundtype))).second;
	}
	else{
		std::cerr << "Sound type " << soundtype << ": not found" << std::endl;
		return;
	}

	result = mSystem->playSound(FMOD_CHANNEL_REUSE, sound, false, &mChannels[soundtype]);

	mChannels[soundtype]->setPaused(true);
	mChannels[soundtype]->setVolume(mVolumes[soundtype]);
	toggleMute();
	toggleMute();
	mChannels[soundtype]->setPaused(false);
	if (soundtype == SoundInterface::Sounds::MusicBad) { mChannels[SoundInterface::Sounds::MusicGood]->setPaused(true); }
	if (soundtype == SoundInterface::Sounds::MusicGood) { mChannels[SoundInterface::Sounds::MusicBad]->setPaused(true); }
	
	ERRCHECK(result);
	mSystem->update();
}

void SoundInterface::stop(SoundInterface::Sounds soundtype)
{
	mChannels[soundtype]->setPaused(true);
}

void SoundInterface::changeVolume(SoundInterface::Sounds soundtype, float volume)
{
	volume = glm::clamp(volume, 0.0f, 1.0f);
	mVolumes[soundtype] = volume;
	mChannels[soundtype]->setVolume(volume);
}

void SoundInterface::toggleMute()
{
	mIsMute = !mIsMute;
	for(int i = 0; i < SoundInterface::Sounds::SOUNDS_COUNT ; i++)
	{
		mChannels[i]->setMute(mIsMute);
	}
}

bool SoundInterface::isPlaying(SoundInterface::Sounds soundtype)
{
	bool isPlaying;
	mChannels[soundtype]->isPlaying(&isPlaying);
	return isPlaying;
}



