#include <fstream>

#include "shader.hpp"
#include "utility.hpp"

	Shader::Shader()
	{
	
	}

	Shader::Shader(const string &path) : _success(false)
	{
		// Load the shader files

		string vertex_shader_source;
		if (fileExists(path+".vert")) {
			vertex_shader_source = readFile(path+".vert");
		} else {
			cerr << "Vertex shader file "
				<< path <<".vert does not exist." << endl;
			return;
		}

		string fragment_shader_source;
		if (fileExists(path+".frag")) {
			fragment_shader_source = readFile(path+".frag");
		} else {
			cerr << "Fragment shader file "
				<< path <<".frag does not exist." << endl;
			return;
		}

		// Compile the shaders

		_vertex_shader = compile(GL_VERTEX_SHADER, vertex_shader_source);
		if (_vertex_shader == 0)
			return;

		getErrors();

		_fragment_shader = compile(GL_FRAGMENT_SHADER, fragment_shader_source);
		if (_fragment_shader == 0)
			return;

		getErrors();

		// Link the shaders into a program

		link();
		if (_program == 0)
			return;

		_success = true;
		getErrors();
	}

	Shader::~Shader()
	{
		glDeleteProgram(_program);
		glDeleteShader(_vertex_shader);
		glDeleteShader(_fragment_shader);
	}

	// Bind the shader to the OGL state-machine
	void Shader::bind() const
	{
		glUseProgram(_program);
	}

	// Unbind the shader
	void Shader::unbind() const
	{
		glUseProgram(0);
	}

	// Query the location of a vertex attribute inside the shader.
	GLint Shader::get_attrib_location(const std::string &name) const 
	{
		return glGetAttribLocation(_program, name.c_str());
	}

	// Query the location of a uniform variable inside the shader.
	GLint Shader::get_uniform_location(const std::string &name) const
	{
		return glGetUniformLocation(_program, name.c_str());
	}

	// Define the name of the variable inside the shader which represents the final color for each fragment.
	void Shader::bind_frag_data_location(const std::string &name) 
	{
		if(_program > 0)
		{
			glBindFragDataLocation(_program, 0, name.c_str() );
			this->link();
		}
	}

	// A little cast helper.
	// With this you can simply do "if (shader) {...}" to test if a
	// shader has been compiled successfully.
	Shader::operator bool ()
	{
		return _success;
	}

	GLuint Shader::compile (GLenum type, const string &source)
	{
		// Create shader object

		GLuint shader = glCreateShader(type);

		if (shader == 0) {
			cerr << "Could not create shader object." << endl;
			return 0;
		}

		// Define shader source and compile

		const char* src = source.data();
		int len = source.size();

		glShaderSource(shader, 1, &src, &len);

		glCompileShader(shader);

		// Check for errors

		int status;

		glGetShaderiv(shader, GL_COMPILE_STATUS, &status);

		if (status != GL_TRUE) {
			cout << "Shader compilation failed." << endl;
			shader_log(shader);
		}

		getErrors();

		return shader;    
	}

	void Shader::link(void)
{
	// Create program handle
	_program = glCreateProgram();

	// Attach shaders and link

	glAttachShader(_program, _vertex_shader);
	glAttachShader(_program, _fragment_shader);

	glLinkProgram(_program);

	// Check for problems

	int status;

	glGetProgramiv(_program, GL_LINK_STATUS, &status);

	if (status != GL_TRUE) {
		cout << "Shader linking failed." << endl;
		program_log(_program);

		glDeleteProgram(_program);
		_program = 0;
	}

	getErrors();
}

	#define LOG_BUFFER_SIZE 8096

	void Shader::program_log(GLuint program)
{
	char logBuffer[LOG_BUFFER_SIZE];
	GLsizei length;

	logBuffer[0] = '\0';
	glGetProgramInfoLog(program, LOG_BUFFER_SIZE, &length,logBuffer);

	if (length > 0) {
		cout << logBuffer << endl;
	}
};

	void Shader::shader_log(GLuint shader)
{
	char logBuffer[LOG_BUFFER_SIZE];
	GLsizei length;

	logBuffer[0] = '\0';
	glGetShaderInfoLog(shader, LOG_BUFFER_SIZE, &length,logBuffer);

	if (length > 0) {
		cout << logBuffer << endl;
	}
};
