#include "utility.hpp"
#include "shader.hpp"
#include "Model.hpp"

#pragma pack(1)
typedef struct
{
    GLbyte	identsize;              // Size of ID field that follows header (0)
    GLbyte	colorMapType;           // 0 = None, 1 = paletted
    GLbyte	imageType;              // 0 = none, 1 = indexed, 2 = rgb, 3 = grey, +8=rle
    unsigned short	colorMapStart;          // First colour map entry
    unsigned short	colorMapLength;         // Number of colors
    unsigned char 	colorMapBits;   // bits per palette entry
    unsigned short	xstart;                 // image x origin
    unsigned short	ystart;                 // image y origin
    unsigned short	width;                  // width in pixels
    unsigned short	height;                 // height in pixels
    GLbyte	bits;                   // bits per pixel (8 16, 24, 32)
    GLbyte	descriptor;             // image descriptor
} TGAHEADER;
#pragma pack(8)

bool fileExists(const string &filename)
{
	std::ifstream ifile(filename.c_str());
	return ifile;
}

string readFile(const string &filename)
{
	std::ifstream ifile(filename.c_str());

	return string(std::istreambuf_iterator<char>(ifile),
		std::istreambuf_iterator<char>());
}

void getErrors(void)
{
	GLenum error = glGetError();

	if (error != GL_NO_ERROR) {
		switch (error) {
		case GL_INVALID_ENUM:
			cerr << "GL: enum argument out of range." << endl;
			break;
		case GL_INVALID_VALUE:
			cerr << "GL: Numeric argument out of range." << endl;
			break;
		case GL_INVALID_OPERATION:
			cerr << "GL: Operation illegal in current state." << endl;
			break;
		case GL_INVALID_FRAMEBUFFER_OPERATION:
			cerr << "GL: Framebuffer object is not complete." << endl;
			break;
		case GL_OUT_OF_MEMORY:
			cerr << "GL: Not enough memory left to execute command." << endl;
			break;
		default:
			cerr << "GL: Unknown error." << endl;
		}
	}
}
void printMatrix(glm::mat4 matrix)
{
	matrix = glm::transpose(matrix);
	cout << endl;
	for (int row = 0; row < 4; row++)
	{
		cout << endl;
		for (int col = 0; col < 4; col++)
		{
			cout << (float) matrix[row][col] << " ";
		}
	}
}

void printMatrix(glm::mat3 matrix)
{
	matrix = glm::transpose(matrix);
	cout << endl;
	for (int row = 0; row < 3; row++)
	{
		cout << endl;
		for (int col = 0; col < 3; col++)
		{
			cout << (float) matrix[row][col] << " ";
		}
	}
}

void printVector(glm::vec2 vec)
{
	for (int row = 0; row < 2; row++)
	{
		cout << (float) vec[row] << " ";
	}
	cout << endl;	
}

void printVector(glm::vec3 vec)
{
	for (int row = 0; row < 3; row++)
	{
		cout <<  (float) vec[row] << " ";
	}
	cout << endl;	
}

void printVector(glm::vec4 vec)
{
	for (int row = 0; row < 4; row++)
	{
		cout << (float) vec[row] << " ";
	}
	cout << endl;	
}

void printArray(float *printarray, int arraysize)
{
	for (int i = 0; i < arraysize; i++)
	{
		cout << printarray[i] << " ";
	}
	cout << endl;
}

float randFloat(float a, float b)
{
	return ((b-a)*((float)rand()/RAND_MAX))+a;
}

int randInt(int a, int b)
{
	return (rand() % (b - a)) + a;
}

int sign(float a)
{
	if (a < 0) { return -1; }
	else { return 1; }
}

float* calculateNormals(const float *vertices_in, int arraysize)
{
	float *normals_out = new float[arraysize / 4 * 3];
	for (int i=0,j=0;i<arraysize;i+=12,j+=9)
	{
		glm::vec3 first = glm::vec3(vertices_in[i+4],vertices_in[i+5],vertices_in[i+6]) - glm::vec3(vertices_in[i],vertices_in[i+1],vertices_in[i+2]);
		glm::vec3 second = glm::vec3(vertices_in[i+8],vertices_in[i+9],vertices_in[i+10]) - glm::vec3(vertices_in[i],vertices_in[i+1],vertices_in[i+2]);
		glm::vec3 crossed = glm::cross(first,second); 
		crossed = glm::normalize(crossed);
		normals_out[j] = normals_out[j+3] = normals_out[j+6] = crossed.x;
		normals_out[j+1] = normals_out[j+4] = normals_out[j+7] = crossed.y;
		normals_out[j+2] = normals_out[j+5] = normals_out[j+8] = crossed.z;
		/*cout << "\r\nFirst:";
		printVector(first);
		cout << "\r\nSecond:";
		printVector(second);
		cout << "\r\nCrossed:";
		printVector(crossed);
		*/
	}
	return normals_out;
}

////////////////////////////////////////////////////////////////////
// Allocate memory and load targa bits. Returns pointer to new buffer,
// height, and width of texture, and the OpenGL format of data.
// Call free() on buffer when finished!
// This only works on pretty vanilla targas... 8, 24, or 32 bit color
// only, no palettes, no RLE encoding.
GLbyte *gltReadTGABits(const char *szFileName, GLint *iWidth, GLint *iHeight, GLint *iComponents, GLenum *eFormat)
   {
   FILE          *pFile;        // File pointer
   TGAHEADER     tgaHeader;     // TGA file header
   unsigned long lImageSize;    // Size in bytes of image
   short         sDepth;        // Pixel depth;
   GLbyte        *pBits = NULL; // Pointer to bits

   // Default/Failed values
   *iWidth = 0;
   *iHeight = 0;
   *eFormat = GL_RGB;
   *iComponents = GL_RGB;

   // Attempt to open the file
   pFile = fopen(szFileName, "rb");
   if(pFile == NULL)
      return NULL;

   // Read in header (binary)
   fread(&tgaHeader, 18/* sizeof(TGAHEADER)*/, 1, pFile);

   // Do byte swap for big vs little endian
#ifdef __APPLE__
   LITTLE_ENDIAN_WORD(&tgaHeader.colorMapStart);
   LITTLE_ENDIAN_WORD(&tgaHeader.colorMapLength);
   LITTLE_ENDIAN_WORD(&tgaHeader.xstart);
   LITTLE_ENDIAN_WORD(&tgaHeader.ystart);
   LITTLE_ENDIAN_WORD(&tgaHeader.width);
   LITTLE_ENDIAN_WORD(&tgaHeader.height);
#endif

   // Get width, height, and depth of texture
   *iWidth = tgaHeader.width;
   *iHeight = tgaHeader.height;
   sDepth = tgaHeader.bits / 8;

   // Put some validity checks here. Very simply, I only understand
   // or care about 8, 24, or 32 bit targa's.
   if(tgaHeader.bits != 8 && tgaHeader.bits != 24 && tgaHeader.bits != 32)
      return NULL;

   // Calculate size of image buffer
   lImageSize = tgaHeader.width * tgaHeader.height * sDepth;

   // Allocate memory and check for success
   pBits = (GLbyte*)malloc(lImageSize * sizeof(GLbyte));
   if(pBits == NULL)
      return NULL;

   // Read in the bits
   // Check for read error. This should catch RLE or other
   // weird formats that I don't want to recognize
   if(fread(pBits, lImageSize, 1, pFile) != 1)
   {
   free(pBits);
   return NULL;
   }

  // Set OpenGL format expected
  switch(sDepth)
     {
			 case 3:     // Most likely case
				 *eFormat = GL_BGR;
				 *iComponents = GL_RGB;
				 break;
			 case 4:
				 *eFormat = GL_BGRA;
				 *iComponents = GL_RGBA;
				 break;
				 default:       // RGB
				// If on the iPhone, TGA's are BGR, and the iPhone does not
				// support BGR without alpha, but it does support RGB,
				// so a simple swizzle of the red and blue bytes will suffice.
				// For faster iPhone loads however, save your TGA's with an Alpha!
				 break;
      }

   // Done with File
   fclose(pFile);

   // Return pointer to image data
   return pBits;
   }

GLuint load2dTexture(const string& path, bool projection)
{
    GLbyte *pBytes;
    GLint iWidth, iHeight, iComponents;
    GLenum eFormat;

    GLuint textureHandle;

    glGenTextures(1, &textureHandle);
    glBindTexture(GL_TEXTURE_2D, textureHandle);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);	
    
	if (projection)
	{
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_BORDER);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_BORDER);
		float color[4] = {0.0f, 0.0f, 0.0f, 1.0f};
		glTexParameterfv(GL_TEXTURE_2D, GL_TEXTURE_BORDER_COLOR, color);
	}
	else
	{	
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);	
	}
    glPixelStorei(GL_UNPACK_ALIGNMENT, 2);
	
	GLfloat fLargest;
	glGetFloatv(GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT, &fLargest);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, fLargest);
	
	// Load this texture map
    pBytes = gltReadTGABits(path.c_str(), &iWidth, &iHeight, &iComponents, &eFormat);
    if(pBytes==NULL) { std::cerr << "image load failed: "<< path << std::endl; }
    glTexImage2D(GL_TEXTURE_2D, 0, iComponents, iWidth, iHeight, 0, eFormat, GL_UNSIGNED_BYTE, pBytes);
    free(pBytes);

    //generate mipmaps
	glGenerateMipmap(GL_TEXTURE_2D);
    //unbind the texture
    glBindTexture(GL_TEXTURE_2D, 0);

    return textureHandle;
}

void setupCubeMap(GLuint& texture) 
{
	glActiveTexture(GL_TEXTURE0);
	//glEnable(GL_TEXTURE_CUBE_MAP); //deprecated	
	glGenTextures(1, &texture);
	glBindTexture(GL_TEXTURE_CUBE_MAP, texture);
	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR); 
	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_S, GL_MIRRORED_REPEAT);
	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_T, GL_MIRRORED_REPEAT);
	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_R, GL_MIRRORED_REPEAT);


}

GLuint loadCubeMap(string paths[]) 
{
	GLuint textureHandle;
	GLbyte *pBytes;
    GLint iWidth, iHeight, iComponents;
    GLenum eFormat;
	int cubeMapEnums[] = {GL_TEXTURE_CUBE_MAP_POSITIVE_X, GL_TEXTURE_CUBE_MAP_NEGATIVE_X, GL_TEXTURE_CUBE_MAP_POSITIVE_Y, GL_TEXTURE_CUBE_MAP_NEGATIVE_Y, GL_TEXTURE_CUBE_MAP_POSITIVE_Z, GL_TEXTURE_CUBE_MAP_NEGATIVE_Z};
	setupCubeMap(textureHandle);
	for (int i=0;i<6;i++)
	{
		// Load this texture map
		pBytes = gltReadTGABits(paths[i].c_str(), &iWidth, &iHeight, &iComponents, &eFormat);
		if(pBytes==NULL) { std::cerr << "image load failed: "<< paths[i] << std::endl; }
		glTexImage2D(cubeMapEnums[i], 0, iComponents, iWidth, iHeight, 0, eFormat, GL_UNSIGNED_BYTE, pBytes);
		free(pBytes);
	}
	glGenerateMipmap(GL_TEXTURE_CUBE_MAP);
	return textureHandle;
}

void unloadCubeMap(GLuint& texture) 
{
	glDeleteTextures(1, &texture);
}

void initVboVao(const Shader& shader, GLuint *vbo_id, GLuint *vao_id, const GLfloat *positions, GLfloat *normals, const GLfloat *texcoordinates, int sizeofarray, int buffermode, bool update)
{
	//PFNGLGENVERTEXARRAYSPROC my_glGenVertexArrays = (PFNGLGENVERTEXARRAYSPROC)glfwGetProcAddress("glGenVertexArrays");
	//PFNGLBINDVERTEXARRAYPROC my_glBindVertexArray = (PFNGLBINDVERTEXARRAYPROC)glfwGetProcAddress("glBindVertexArray");

	if (!update)
	{
		// Allocate and assign a Vertex Array Object to our handle
		glGenVertexArrays(1, vao_id);
		//my_glGenVertexArrays(1, vao_id);
	}
	// Bind our Vertex Array Object as the current used object
	glBindVertexArray(*vao_id);
	//my_glBindVertexArray(*vao_id);

	if (!update)
	{
		// Allocate and assign the Vertex Buffer Objects (VBOs) to our handle
		glGenBuffers(3, vbo_id);
	}
	// Bind our first VBO as being the active buffer and storing vertex attributes (coordinates)	
	glBindBuffer(GL_ARRAY_BUFFER, vbo_id[0]);
	glBufferData(GL_ARRAY_BUFFER, sizeofarray, positions, buffermode);

	GLint vertex_location = shader.get_attrib_location("vVertex");
	glEnableVertexAttribArray(vertex_location);
	glVertexAttribPointer(	vertex_location, 4, GL_FLOAT, GL_FALSE, 0, NULL);

	if (normals != NULL) 
	{
		// Bind our second VBO as being the active buffer and storing vertex normals 	
		glBindBuffer(GL_ARRAY_BUFFER, vbo_id[1]);
		glBufferData(GL_ARRAY_BUFFER, sizeofarray / 4 * 3, normals, buffermode);
 
		GLint normal_location = shader.get_attrib_location("vNormal");
		glEnableVertexAttribArray(normal_location);
		glVertexAttribPointer(	normal_location, 3, GL_FLOAT, GL_FALSE, 0, NULL);
	}

	if (texcoordinates != NULL) 
	{
		// Bind our third VBO as being the active buffer and storing texture coords 	
		glBindBuffer(GL_ARRAY_BUFFER, vbo_id[2]);
		glBufferData(GL_ARRAY_BUFFER, sizeofarray / 2, texcoordinates, buffermode);
 
		GLint texcoordinates_location = shader.get_attrib_location("vTexCoord");
		glEnableVertexAttribArray(texcoordinates_location);
		glVertexAttribPointer(	texcoordinates_location, 2, GL_FLOAT, GL_FALSE, 0, NULL);
	}
	
	// unbind VAO
	glBindVertexArray(0);
	//my_glBindVertexArray(0);

	// Unbind VBOs
	glBindBuffer(GL_ARRAY_BUFFER, 0);
}

//void initVboVaoFromModel(const Shader &shader, GLuint *vbo_id, GLuint *vao_id, const string& path, int buffermode, int *numberofmeshes, int **out_numberofverticespermesh)
void initVboVaoFromModel(const Shader &shader, Model *model, const string& path, int buffermode)
{
	double timeBeforeLoad = glfwGetTime();

	Assimp::Importer importer;
	const aiScene* scene = importer.ReadFile( path,
		aiProcess_CalcTangentSpace |
		aiProcess_Triangulate |
//		aiProcess_JoinIdenticalVertices |
		aiProcess_FlipUVs
	);

	if(scene->HasMeshes()) 
	{
		//calculate necessary size
		int necessarysize = 0;
		for (UINT k=0; k < scene->mNumMeshes;k++) 
		{ 
			necessarysize += scene->mMeshes[k]->mNumVertices * 4; 
		}		
		model->mNumberOfVerticesPerMesh = new int[scene->mNumMeshes];
		float *vertices = new float[necessarysize];
		float *normals = new float[necessarysize / 4 * 3];
		float *texcoords = new float[necessarysize / 4 * 2];
		//sStdGeomVertex* vertices = new sStdGeomVertex[scene->mMeshes[0]->mNumVertices];
		model->mNumberOfMeshes = (int)scene->mNumMeshes;
		int offsetvertices = 0, offsetnormals = 0, offsettexcoords = 0;

		for (UINT k=0; k < scene->mNumMeshes;k++)
		{
			model->mNumberOfVerticesPerMesh[k] = scene->mMeshes[k]->mNumVertices;
			for(UINT i=0, vertexstep=0, normalstep=0, texstep = 0; i < scene->mMeshes[k]->mNumVertices;i++,vertexstep+=4,normalstep+=3,texstep+=2)
			{
				vertices[offsetvertices + (vertexstep+0)] = scene->mMeshes[k]->mVertices[i].x;
				vertices[offsetvertices + (vertexstep+1)] = scene->mMeshes[k]->mVertices[i].y;
				vertices[offsetvertices + (vertexstep+2)] = scene->mMeshes[k]->mVertices[i].z;
				vertices[offsetvertices + (vertexstep+3)] = 1;
				
				if (scene->mMeshes[k]->HasNormals()) 
				{
					normals[offsetnormals + (normalstep+0)] = scene->mMeshes[k]->mNormals[i].x;
					normals[offsetnormals + (normalstep+1)] = scene->mMeshes[k]->mNormals[i].y;
					normals[offsetnormals + (normalstep+2)] = scene->mMeshes[k]->mNormals[i].z;
				}
				if (scene->mMeshes[k]->HasTextureCoords(0))
				{
					//if (path.find("stonewall") != string::npos)
					//{
					//	static int counter = 0;
					//	if (counter > 5) counter = 0;
					//	if (counter == 0) { texcoords[offsettexcoords + (texstep+0)] = 0; texcoords[offsettexcoords + (texstep+1)] = 0;}
					//	if (counter == 1) { texcoords[offsettexcoords + (texstep+0)] = 10; texcoords[offsettexcoords + (texstep+1)] = 0;}
					//	if (counter == 2) { texcoords[offsettexcoords + (texstep+0)] = 10; texcoords[offsettexcoords + (texstep+1)] = 1;}						
					//	if (counter == 3) { texcoords[offsettexcoords + (texstep+0)] = 0; texcoords[offsettexcoords + (texstep+1)] = 0;}
					//	if (counter == 4) { texcoords[offsettexcoords + (texstep+0)] = 10; texcoords[offsettexcoords + (texstep+1)] = 1;}
					//	if (counter == 5) { texcoords[offsettexcoords + (texstep+0)] = 0; texcoords[offsettexcoords + (texstep+1)] = 1;}
					//	counter++;
					//	
					//}
					//else
					//{
						texcoords[offsettexcoords + (texstep+0)] = scene->mMeshes[k]->mTextureCoords[0][i].x;
						texcoords[offsettexcoords + (texstep+1)] = scene->mMeshes[k]->mTextureCoords[0][i].y;
					//}
				}
			}
			offsetvertices += model->mNumberOfVerticesPerMesh[k] * 4;
			offsetnormals += model->mNumberOfVerticesPerMesh[k] * 3;
			offsettexcoords += model->mNumberOfVerticesPerMesh[k] * 2;
		}

		double timeAfterLoad = glfwGetTime();
		cout << "Loading: " + path + " needed time: " << timeAfterLoad - timeBeforeLoad << " mesh count: " << scene->mNumMeshes << " vertices count: " << necessarysize << endl;
		PFNGLGENVERTEXARRAYSPROC my_glGenVertexArrays = (PFNGLGENVERTEXARRAYSPROC)glfwGetProcAddress("glGenVertexArrays");
		PFNGLBINDVERTEXARRAYPROC my_glBindVertexArray = (PFNGLBINDVERTEXARRAYPROC)glfwGetProcAddress("glBindVertexArray");

		// Allocate and assign a Vertex Array Object to our handle
		my_glGenVertexArrays(1, &model->mVaoId);

		// Bind our Vertex Array Object as the current used object
		my_glBindVertexArray(model->mVaoId);

		// Allocate and assign three Vertex Buffer Objects (VBOs) to our handle
		glGenBuffers(3, model->mVboId);

		// Bind our first VBO as being the active buffer and storing vertex attributes (coordinates)	
		glBindBuffer(GL_ARRAY_BUFFER,  model->mVboId[0]);
		glBufferData(GL_ARRAY_BUFFER, offsetvertices * sizeof(float), vertices, buffermode);

		GLint vertex_location = shader.get_attrib_location("vVertex");
		glEnableVertexAttribArray(vertex_location);
		glVertexAttribPointer(	vertex_location, 4, GL_FLOAT, GL_FALSE, 0, NULL);

		// Bind our second VBO as being the active buffer and storing vertex normals 	
		glBindBuffer(GL_ARRAY_BUFFER,  model->mVboId[1]);
		glBufferData(GL_ARRAY_BUFFER, offsetnormals * sizeof(float), normals, buffermode);
 
		GLint normal_location = shader.get_attrib_location("vNormal");
		glEnableVertexAttribArray(normal_location);
		glVertexAttribPointer(	normal_location, 3, GL_FLOAT, GL_FALSE, 0, NULL);

		// Bind our third VBO as being the active buffer and storing texture coords 	
		glBindBuffer(GL_ARRAY_BUFFER,  model->mVboId[2]);
		glBufferData(GL_ARRAY_BUFFER, offsettexcoords * sizeof(float), texcoords, buffermode);
 
		GLint texcoordinates_location = shader.get_attrib_location("vTexCoord");
		glEnableVertexAttribArray(texcoordinates_location);
		glVertexAttribPointer(	texcoordinates_location, 2, GL_FLOAT, GL_FALSE, 0, NULL);
		
		//delete allocated mem
		delete(vertices);
		delete(normals);
		delete(texcoords);

		// unbind VAO
		my_glBindVertexArray(0);

		// Unbind VBOs
		glBindBuffer(GL_ARRAY_BUFFER, 0);
		//*out_numberofverticespermesh = numberofverticespermesh;
	}
}

void releaseVboVao(GLuint *vbo_id, GLuint *vao_id)
{
	//PFNGLDELETEVERTEXARRAYSPROC my_glDeleteVertexArrays = (PFNGLDELETEVERTEXARRAYSPROC)glfwGetProcAddress("glDeleteVertexArrays");
	//my_glDeleteVertexArrays(1, vao_id);

	//glDeleteBuffers(3, vbo_id);

	//*vao_id = 0;
	//vbo_id[0] = vbo_id[1] = vbo_id[2] = 0;
}



bool initFbo(GLuint &fboId, GLuint &imgId, int width, int height)
{
	// generate fbo
	glGenFramebuffers(1, &fboId); 
	// bind FBO
	glBindFramebuffer(GL_FRAMEBUFFER, fboId);


	GLuint depthbuffer; // this will store our db-name
	// create a depth-buffer
	glGenRenderbuffers(1, &depthbuffer);
	// bind our depth-buffer
	glBindRenderbuffer(GL_RENDERBUFFER, depthbuffer);
	// create storage for our renderbuffer
	glRenderbufferStorage(GL_RENDERBUFFER, GL_DEPTH_COMPONENT, width, height);
	// attach renderbuffer to FBO
	glFramebufferRenderbuffer(GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, GL_RENDERBUFFER, depthbuffer);

	// create a texture
	glGenTextures(1, &imgId);
	

	glBindTexture(GL_TEXTURE_2D, imgId);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_R32I, width, height, 0, GL_RED_INTEGER, GL_INT, NULL);
	// attach texture to fbo
	glFramebufferTexture2D(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, GL_TEXTURE_2D, imgId, 0);
	// fbo-creation error-checking
	GLenum status = glCheckFramebufferStatus(GL_FRAMEBUFFER);
	if (status != GL_FRAMEBUFFER_COMPLETE)
	{
		cout << "fbo init faiiiled!!! you are dooomed!!";
		return false;
	}
	glBindFramebuffer(GL_FRAMEBUFFER, 0);

	return true;
}


int readPixel(unsigned int x, unsigned int y, GLuint fboId, GLuint imgId)
{
	glBindFramebuffer(GL_READ_FRAMEBUFFER, fboId);
    glReadBuffer(GL_COLOR_ATTACHMENT0);
	//glBindTexture(GL_TEXTURE_2D, imgId);
	int objectid = 0;
    glReadPixels(x, y, 1, 1, GL_RED_INTEGER, GL_INT, &objectid);
	//cout << objectid << endl;
    glReadBuffer(GL_NONE);
    glBindFramebuffer(GL_READ_FRAMEBUFFER, 0);
	//glBindTexture(GL_TEXTURE_2D, 0);
    return objectid;
} 

int *readPixels(unsigned int xFrom, unsigned int yFrom, unsigned int width, unsigned int height, GLuint fboId, GLuint imgId)
{
	glBindFramebuffer(GL_READ_FRAMEBUFFER, fboId);
    glReadBuffer(GL_COLOR_ATTACHMENT0);
	//glBindTexture(GL_TEXTURE_2D, imgId);
	//allocate memory for pixel data
	int *objectIDs = new int[width * height];
    glReadPixels(xFrom, yFrom, width, height, GL_RED_INTEGER, GL_INT, objectIDs);
	//cout << objectid << endl;
    glReadBuffer(GL_NONE);
    glBindFramebuffer(GL_READ_FRAMEBUFFER, 0);
	//glBindTexture(GL_TEXTURE_2D, 0);
    return objectIDs;
} 
